import click

# The exit(1) is used to indicate error in pre-commit
NOT_OK = 1


@click.command()
@click.option("--filename1", default="requirements.txt")
@click.option("--filename2", default="requirements-private.txt")
def rqf(filename1, filename2):
    requirements = open_file(filename1)
    private_txt = open_file(filename2)

    at_sign_set = create_set(private_txt, "@")
    requirements_without_at_sign = remove_common_elements(
        requirements, at_sign_set
    )

    # the filename1 will be overwritten
    write_file(filename1, requirements_without_at_sign)


def remove_common_elements(package_list, remove_set):
    """
    Remove the common elements between package_list and remove_set.

    Note that this is *not* an XOR operation: packages that do not
    exist in remove_set (but exists in remove_set) are not included.

    Parameters
    ----------
    package_list : list
        List with string elements representing the packages from the
        requirements file. Assumes that the list has "==" to denote
        package versions.

    remove_set : set
        Set with the names of packages to be removed from requirements.

    Returns
    -------
    list
        List of packages not presented in remove_set.
    """
    package_not_in_remove_set = []
    for package in package_list:
        package_name = package.split("==")[0].strip()
        if package_name not in remove_set:
            package_not_in_remove_set.append(package)
    return package_not_in_remove_set


def open_file(filename):
    """
    Open txt file.

    Parameters
    ----------
    filename : str
        Name of the file to be opened.

    Returns
    -------
    list
        List of strings with the packages names and versions.
    """
    try:
        with open(filename) as file_object:
            requirements = file_object.readlines()
        return requirements
    except FileNotFoundError:
        print(f"{filename} not found.")
        exit(NOT_OK)


def create_set(package_list, delimiter):
    """
    Create a set of packages to be excluded.

    This function receives a list of strings, takes the packages' names
    and transforms them to a set.

    If the list contains packages with @ but the delimiter input is '==',
    then the package is ignored.

    Parameters
    ----------
    package_list : list
        List of strings with each element representing a package name
        and version.

    Returns
    -------
    set
        Set with the package names.
    """
    list_of_package_names = []
    for package in package_list:
        if delimiter in package:
            package_name = package.split(delimiter)
            list_of_package_names.append(package_name[0].strip())
    package_set = set(list_of_package_names)
    return package_set


def write_file(filename, information):
    """
    Write string information into a file.

    Parameters
    ----------
    file1 : str
        Name of the file where the information will be saved.
        It should have the filepath and the file extension (.txt).

    information : str
        Information to be saved.
    """
    with open(filename, "w") as file_save:
        for line in information:
            file_save.write(line)


if __name__ == "__main__":
    rqf()



import tempfile
import unittest
from pathlib import Path

from click.testing import CliRunner

from requirements_filter import requirements_filter as rq
from requirements_filter.requirements_filter import NOT_OK

requirements = """
flake8==3.9.0
project==0.0.0
"""


requirements_at_sign = r"""project @ git+https://${GITHUB_TOKEN}@github.com/user/project.git@{version}
"""


requirements_without_private = """
flake8==3.9.0
"""


def convert_str_list(my_string):
    """
    Function to convert from string to list.

    The create_set() function requires a specific input.

    Parameters
    ----------
    my_string : str
        Input string.

    Returns
    -------
    list
        The input string converted to list.
    """
    return my_string.strip("\n").split("\n")


class RequirementsFilterTest(unittest.TestCase):
    def test_true(self):
        self.assertEqual(1, 1)

    def test_rqf_two_files(self):
        runner = CliRunner()
        with tempfile.TemporaryDirectory() as tp:
            rq.write_file(f"{tp}/requirements.txt", requirements)
            rq.write_file(
                f"{tp}/requirements-private.txt", requirements_at_sign
            )
            runner.invoke(
                rq.rqf,
                [
                    "--filename1",
                    f"{tp}/requirements.txt",
                    "--filename2",
                    f"{tp}/requirements-private.txt",
                ],
            )

            with open(Path(f"{tp}/requirements.txt")) as file_object:
                received = file_object.readlines()
            self.assertEqual("".join(received), requirements_without_private)

    def test_open_file_invalid_filename(self):
        invalid_filename = "invalid_requirements.txt"
        with self.assertRaises(SystemExit) as cm:
            rq.open_file(invalid_filename)
        self.assertEqual(cm.exception.code, NOT_OK)

    def test_open_file_valid_filename(self):
        right_filename = "requirements.txt"
        received = rq.open_file(right_filename)
        self.assertIs(type(received), list)

    def test_create_set_is_set(self):
        input_list = convert_str_list(requirements_at_sign)
        received = rq.create_set(input_list, "@")
        self.assertIsInstance(received, set)

    def test_create_set_at_sign(self):
        answer_list = ["project"]
        answer = set(answer_list)
        input_list = convert_str_list(requirements_at_sign)
        received = rq.create_set(input_list, "@")
        self.assertEqual(answer, received)

    def test_create_set_equal_sign(self):
        answer_package_list = ["flake8", "project"]
        answer_set = set(answer_package_list)
        input_list = convert_str_list(requirements)
        received = rq.create_set(input_list, "==")
        self.assertEqual(answer_set, received)

    def test_create_set_at_equal_sign(self):
        # test what happens with packages using @ delimiter
        # are applied with the "==" parameter
        answer_package_list = []
        answer_set = set(answer_package_list)
        input_list = convert_str_list(requirements_at_sign)
        received = rq.create_set(input_list, "==")
        self.assertEqual(answer_set, received)

    def test_remove_common_elements_equal_sign(self):
        input_list = convert_str_list(requirements)
        my_input_set = set(["project"])
        received = rq.remove_common_elements(input_list, my_input_set)
        answer = ["flake8==3.9.0"]
        self.assertEqual(answer, received)


if __name__ == "__main__":
    unittest.main()
